@@ -12,6 +12,7 @@ __copyright__ = '(c) 2012 Canonical Ltd.'
 
 import copy
 import importlib
+import importlib.util
 import os
 import sys
 import time
@@ -46,7 +47,7 @@ def load_module(name: str):
         spec = importlib.util.spec_from_file_location(os.path.splitext(os.path.basename(name))[0], name)
         assert spec
         mod = importlib.util.module_from_spec(spec)
-        with open(name) as f:
+        with open(name, encoding="UTF-8") as f:
             exec(f.read(), mod.__dict__, mod.__dict__)  # pylint: disable=exec-used
         return mod
 
@@ -91,7 +92,6 @@ def _format_args(args):
 
 def _wrap_in_dbus_variant(value):
     dbus_types = [
-        dbus.types.Array,
         dbus.types.ByteArray,
         dbus.types.Int16,
         dbus.types.ObjectPath,
@@ -111,12 +111,13 @@ def _wrap_in_dbus_variant(value):
     ]
     if isinstance(value, dbus.String):
         return dbus.String(str(value), variant_level=1)
+    if isinstance(value, dbus.types.Array):
+        return value
     if type(value) in dbus_types:
         return type(value)(value.conjugate(), variant_level=1)
     if isinstance(value, str):
         return dbus.String(value, variant_level=1)
-    raise dbus.exceptions.DBusException(
-        'could not wrap type {}'.format(type(value)))
+    raise dbus.exceptions.DBusException(f'could not wrap type {type(value)}')
 
 
 class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-attributes
@@ -125,6 +126,9 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     This can be configured to have arbitrary methods (including code execution)
     and properties via methods on the org.freedesktop.DBus.Mock interface, so
     that you can control the mock from any programming language.
+
+    Beyond that "remote control" API, this is a standard dbus-python service object, see
+    <https://dbus.freedesktop.org/doc/dbus-python/tutorial.html#exporting-objects>.
     '''
 
     def __init__(self, bus_name: str, path: str, interface: str, props: PropsType,
@@ -178,7 +182,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         if self.path == '/':
             cond = 'k != \'/\''
         else:
-            cond = 'k.startswith(\'%s/\')' % self.path
+            cond = f'k.startswith(\'{self.path}/\')'
 
         self.AddMethod(OBJECT_MANAGER_IFACE,
                        'GetManagedObjects', '', 'a{oa{sa{sv}}}',
@@ -201,7 +205,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     def Get(self, interface_name: str, property_name: str) -> Any:
         '''Standard D-Bus API for getting a property value'''
 
-        self.log('Get %s.%s' % (interface_name, property_name))
+        self.log(f'Get {self.path} {interface_name}.{property_name}')
 
         if not interface_name:
             interface_name = self.interface
@@ -217,7 +221,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     def GetAll(self, interface_name: str, *_, **__) -> PropsType:
         '''Standard D-Bus API for getting all property values'''
 
-        self.log('GetAll ' + interface_name)
+        self.log(f'GetAll {self.path} {interface_name}')
 
         if not interface_name:
             interface_name = self.interface
@@ -233,9 +237,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
     def Set(self, interface_name: str, property_name: str, value: Any, *_, **__) -> None:
         '''Standard D-Bus API for setting a property value'''
 
-        self.log('Set %s.%s%s' % (interface_name,
-                                  property_name,
-                                  _format_args((value,))))
+        self.log(f'Set {self.path} {interface_name}.{property_name}{_format_args((value,))}')
 
         try:
             iface_props = self.props[interface_name]
@@ -263,7 +265,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                          in_signature='ssa{sv}a(ssss)',
                          out_signature='')
     def AddObject(self, path: str, interface: str, properties: PropsType, methods: List[MethodType]) -> None:
-        '''Add a new D-Bus object to the mock
+        '''Dynamically add a new D-Bus object to the mock
 
         path: D-Bus object path
         interface: Primary D-Bus interface name of this object (where
@@ -292,9 +294,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                              ])
         '''
         if path in objects:
-            raise dbus.exceptions.DBusException(
-                'object %s already exists' % path,
-                name='org.freedesktop.DBus.Mock.NameError')
+            raise dbus.exceptions.DBusException(f'object {path} already exists', name='org.freedesktop.DBus.Mock.NameError')
 
         obj = DBusMockObject(self.bus_name,
                              path,
@@ -322,7 +322,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
             del objects[path]
         except KeyError as e:
             raise dbus.exceptions.DBusException(
-                'object %s does not exist' % path,
+                f'object {path} does not exist',
                 name='org.freedesktop.DBus.Mock.NameError') from e
 
     @dbus.service.method(MOCK_IFACE,
@@ -362,7 +362,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                          in_signature='sssss',
                          out_signature='')
     def AddMethod(self, interface, name: str, in_sig: str, out_sig: str, code: str) -> None:
-        '''Add a method to this object
+        '''Dynamically add a method to this object
 
         interface: D-Bus interface to add this to. For convenience you can
                    specify '' here to add the method to the object's main
@@ -386,8 +386,20 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
               "self.my_state = True", and another method that returns it with
               "ret = self.my_state".
 
+              Methods can raise exceptions in the usual way, in particular
+              dbus.exceptions.DBusException:
+              <https://dbus.freedesktop.org/doc/dbus-python/dbus.html#dbus.DBusException>
+
               When specifying '', the method will not do anything (except
               logging) and return None.
+
+
+        This is meant for adding a method to a mock at runtime, from any programming language.
+        You can also use it in templates in the load() function.
+
+        For implementing non-trivial and static methods in templates, it is recommended to
+        implement them in the normal dbus-python way with using the @dbus.service.method
+        decorator instead.
         '''
         # pylint: disable=protected-access
 
@@ -408,7 +420,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
                                           out_signature=out_sig)(method)
         dbus_method.__name__ = str(name)
         dbus_method._dbus_in_signature = in_sig
-        dbus_method._dbus_args = ['arg%i' % i for i in range(1, n_args + 1)]
+        dbus_method._dbus_args = [f'arg{i}' for i in range(1, n_args + 1)]
 
         # for convenience, add mocked methods on the primary interface as
         # callable methods
@@ -458,9 +470,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
             if not interface:
                 interface = self.interface
             if name not in self.props.get(interface, {}):
-                raise dbus.exceptions.DBusException(
-                    'property %s not found' % name,
-                    name=interface + '.NoSuchProperty')
+                raise dbus.exceptions.DBusException(f'property {name} not found', name=interface + '.NoSuchProperty')
 
             self._set_property(interface, name, value)
             changed_props[name] = _wrap_in_dbus_variant(value)
@@ -483,9 +493,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         if not interface:
             interface = self.interface
         if name in self.props.get(interface, {}):
-            raise dbus.exceptions.DBusException(
-                'property %s already exists' % name,
-                name=self.interface + '.PropertyExists')
+            raise dbus.exceptions.DBusException(f'property {name} already exists', name=self.interface + '.PropertyExists')
 
         self._set_property(interface, name, value)
 
@@ -528,7 +536,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         try:
             module = load_module(template)
         except ImportError as e:
-            raise dbus.exceptions.DBusException('Cannot add template %s: %s' % (template, str(e)),
+            raise dbus.exceptions.DBusException(f'Cannot add template {template}: {str(e)}',
                                                 name='org.freedesktop.DBus.Mock.TemplateError')
 
         # If the template specifies this is an ObjectManager, set that up
@@ -585,11 +593,11 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         m.append(signature=signature, *args)
         args = m.get_args_list()
 
-        fn = lambda self, *args: self.log('emit %s.%s%s' % (interface, name, _format_args(args)))
+        fn = lambda self, *args: self.log(f'emit {self.path} {interface}.{name}{_format_args(args)}')
         fn.__name__ = str(name)
         dbus_fn = dbus.service.signal(interface)(fn)
         dbus_fn._dbus_signature = signature
-        dbus_fn._dbus_args = ['arg%i' % i for i in range(1, len(args) + 1)]
+        dbus_fn._dbus_args = [f'arg{i}' for i in range(1, len(args) + 1)]
 
         dbus_fn(self, *args)
 
@@ -692,7 +700,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
         else:
             fd = sys.stdout.fileno()
 
-        os.write(fd, ('%.3f %s\n' % (time.time(), msg)).encode('UTF-8'))
+        os.write(fd, f'{time.time():.3f} {msg}\n'.encode('UTF-8'))
 
     @dbus.service.method(dbus.INTROSPECTABLE_IFACE,
                          in_signature='',
@@ -726,7 +734,7 @@ class DBusMockObject(dbus.service.Object):  # pylint: disable=too-many-instance-
             # We might have properties for new interfaces we don't know about
             # yet. Try to find an existing <interface> node named after our
             # interface to append to, and create one if we can't.
-            interface = tree.find(".//interface[@name='%s']" % name)
+            interface = tree.find(f".//interface[@name='{name}']")
             if interface is None:
                 interface = ElementTree.Element("interface", {"name": name})
                 tree.append(interface)
