@@ -134,6 +134,16 @@ assert args[2] == 5
         # check that it's logged correctly
         self.assertLog(b'^[0-9.]+ Do -1 {"foo": 42} 5$')
 
+    def test_exception(self):
+        '''raise a D-Bus exception'''
+
+        self.dbus_mock.AddMethod('', 'Do', '', 'i',
+                                 'raise dbus.exceptions.DBusException("no good", name="com.example.Error.NoGood")')
+        with self.assertRaises(dbus.exceptions.DBusException) as cm:
+            self.dbus_test.Do()
+        self.assertEqual(cm.exception.get_dbus_name(), 'com.example.Error.NoGood')
+        self.assertEqual(cm.exception.get_dbus_message(), 'no good')
+
     def test_methods_on_other_interfaces(self):
         '''methods on other interfaces'''
 
@@ -318,10 +328,10 @@ assert args[2] == 5
         # check that the Get/Set calls get logged
         with open(self.mock_log.name, encoding="UTF-8") as f:
             contents = f.read()
-            self.assertRegex(contents, '\n[0-9.]+ Get org.freedesktop.Test.Main.version\n')
-            self.assertRegex(contents, '\n[0-9.]+ Get org.freedesktop.Test.Main.connected\n')
-            self.assertRegex(contents, '\n[0-9.]+ GetAll org.freedesktop.Test.Main\n')
-            self.assertRegex(contents, '\n[0-9.]+ Set org.freedesktop.Test.Main.version 4\n')
+            self.assertRegex(contents, '\n[0-9.]+ Get / org.freedesktop.Test.Main.version\n')
+            self.assertRegex(contents, '\n[0-9.]+ Get / org.freedesktop.Test.Main.connected\n')
+            self.assertRegex(contents, '\n[0-9.]+ GetAll / org.freedesktop.Test.Main\n')
+            self.assertRegex(contents, '\n[0-9.]+ Set / org.freedesktop.Test.Main.version 4\n')
 
         # add property to different interface
         self.dbus_mock.AddProperty('org.freedesktop.Test.Other',
@@ -512,10 +522,10 @@ assert args[2] == 5
         # check correct logging
         with open(self.mock_log.name, encoding="UTF-8") as f:
             log = f.read()
-        self.assertRegex(log, '[0-9.]+ emit org.freedesktop.Test.Main.SigNoArgs\n')
-        self.assertRegex(log, '[0-9.]+ emit org.freedesktop.Test.Sub.SigTwoArgs "hello" 42\n')
-        self.assertRegex(log, '[0-9.]+ emit org.freedesktop.Test.Sub.SigTypeTest -42 42')
-        self.assertRegex(log, r'[0-9.]+ emit org.freedesktop.Test.Sub.SigTypeTest -42 42 "hello" \["/a", "/b"\]\n')
+        self.assertRegex(log, '[0-9.]+ emit / org.freedesktop.Test.Main.SigNoArgs\n')
+        self.assertRegex(log, '[0-9.]+ emit / org.freedesktop.Test.Sub.SigTwoArgs "hello" 42\n')
+        self.assertRegex(log, '[0-9.]+ emit / org.freedesktop.Test.Sub.SigTypeTest -42 42')
+        self.assertRegex(log, r'[0-9.]+ emit / org.freedesktop.Test.Sub.SigTypeTest -42 42 "hello" \["/a", "/b"\]\n')
 
     def test_signals_type_mismatch(self):
         '''emitting signals with wrong arguments'''
